/**
 *  Copyright (c) 2015, Facebook, Inc.
 *  All rights reserved.
 *
 *  This source code is licensed under the license found in the
 *  LICENSE-examples file in the root directory of this source tree.
 */

'use strict';

exports.__esModule = true;

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

function _inherits(subClass, superClass) { if (typeof superClass !== 'function' && superClass !== null) { throw new TypeError('Super expression must either be null or a function, not ' + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _reactDom2 = _interopRequireDefault(_reactDom);

var _codemirror = require('codemirror');

var _codemirror2 = _interopRequireDefault(_codemirror);

require('codemirror/addon/fold/foldgutter');

require('codemirror/addon/fold/brace-fold');

require('codemirror/keymap/sublime');

require('codemirror/mode/javascript/javascript');

/**
 * ResultViewer
 *
 * Maintains an instance of CodeMirror for viewing a GraphQL response.
 *
 * Props:
 *
 *   - value: The text of the editor.
 *
 */

var ResultViewer = (function (_React$Component) {
  _inherits(ResultViewer, _React$Component);

  function ResultViewer() {
    _classCallCheck(this, ResultViewer);

    _React$Component.apply(this, arguments);
  }

  ResultViewer.prototype.componentDidMount = function componentDidMount() {
    this.viewer = _codemirror2['default'](_reactDom2['default'].findDOMNode(this), {
      value: this.props.value || '',
      readOnly: true,
      theme: 'graphiql',
      mode: {
        name: 'javascript',
        json: true
      },
      keyMap: 'sublime',
      foldGutter: {
        minFoldSize: 4
      },
      gutters: ['CodeMirror-foldgutter'],
      extraKeys: {
        // Editor improvements
        'Ctrl-Left': 'goSubwordLeft',
        'Ctrl-Right': 'goSubwordRight',
        'Alt-Left': 'goGroupLeft',
        'Alt-Right': 'goGroupRight'
      }
    });
  };

  ResultViewer.prototype.componentWillUnmount = function componentWillUnmount() {
    this.viewer = null;
  };

  ResultViewer.prototype.shouldComponentUpdate = function shouldComponentUpdate(nextProps) {
    return this.props.value !== nextProps.value;
  };

  ResultViewer.prototype.componentDidUpdate = function componentDidUpdate() {
    this.viewer.setValue(this.props.value || '');
  };

  ResultViewer.prototype.render = function render() {
    return _react2['default'].createElement('div', { className: 'result-window' });
  };

  _createClass(ResultViewer, null, [{
    key: 'propTypes',
    value: {
      value: _react.PropTypes.string
    },
    enumerable: true
  }]);

  return ResultViewer;
})(_react2['default'].Component);

exports.ResultViewer = ResultViewer;